<?php
namespace FiloBlu\Storelocator\Controller\Adminhtml\Stores;

use Magento\Backend\App\Action;
use Magento\TestFramework\ErrorLog\Logger;

class Save extends \Magento\Backend\App\Action
{

    protected $uploaderFactory;
    protected $imageModel;
    protected $jsHelper;

    /**
     * @param Action\Context $context
     */
    public function __construct(
        Action\Context $context,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploaderFactory,
        \FiloBlu\Storelocator\Model\Stores\Image $imageModel,
        \Magento\Backend\Helper\Js $jsHelper
    )
    {
        $this->uploaderFactory = $uploaderFactory;
        $this->imageModel = $imageModel;
        $this->jsHelper = $jsHelper;
        parent::__construct($context);
    }

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_Storelocator::storelocator_save');
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        if ($data) {

            /** @var \FiloBlu\Storelocator\Model\Stores $model */
            $model = $this->_objectManager->create('FiloBlu\Storelocator\Model\Stores');

            $id = $this->getRequest()->getParam('store_id');

            if ($id) {
                $model->load($id);
            }

            $model->setData($data);

            $imageName = $this->uploadFileAndGetName('image', $this->imageModel->getBaseDir(), $data);
            $model->setImage($imageName);
            $markerImageName = $this->uploadFileAndGetName('marker_image', $this->imageModel->getBaseDir(), $data);
            $model->setMarkerImage($markerImageName);

            if (!$model->getLatitude() || !$model->getLongitude()) {
                $address = $model->getData("street") . " " .
                    $model->getData("postcode") . " " .
                    $model->getData("city") . " " .
                    $model->getData("country");
                $geocodeData = $this->_objectManager->create('FiloBlu\Storelocator\Helper\Data')->geocode($address);
                if ($geocodeData) {
                    $model->setLatitude($geocodeData[0]);
                    $model->setLongitude($geocodeData[1]);
                    $model->save();
                    $this->messageManager->addSuccess(__('Geolocation data has been updated. Address found ' . $geocodeData[2]));
                }
            }

            $tags = $this->getRequest()->getPost('tags', -1);

            if ($tags != -1) {
                $model->setTagsData($this->jsHelper->decodeGridSerializedInput($tags));
            }

            $images = $this->getRequest()->getPost('images', -1);

            if ($images != -1) {
                $model->setImagesData($this->jsHelper->decodeGridSerializedInput($images));
            }

            // Working Times
            $objWorkingTimes = [];
            for ($wt = 1; $wt <= 7; $wt++) {
                $dayStr = '';

                switch ($wt) {
                    case 1:
                        $dayStr = "monday";
                        break;
                    case 2:
                        $dayStr = "tuesday";
                        break;
                    case 3:
                        $dayStr = "wednesday";
                        break;
                    case 4:
                        $dayStr = "thursday";
                        break;
                    case 5:
                        $dayStr = "friday";
                        break;
                    case 6:
                        $dayStr = "saturday";
                        break;
                    case 7:
                        $dayStr = "sunday";
                        break;

                    default:
                        break;
                }

                $objWorkingTimes[$dayStr . '_morning_time_from'] = '01/01/2000 ' . $data[$dayStr . '_morning_time_from'][0] . ':' . $data[$dayStr . '_morning_time_from'][1] . ':' . $data[$dayStr . '_morning_time_from'][2];
                $objWorkingTimes[$dayStr . '_morning_time_to'] = '01/01/2000 ' . $data[$dayStr . '_morning_time_to'][0] . ':' . $data[$dayStr . '_morning_time_to'][1] . ':' . $data[$dayStr . '_morning_time_to'][2];
                $objWorkingTimes['closed_' . $dayStr . '_morning'] = $data['closed_' . $dayStr . '_morning'];

                $objWorkingTimes[$dayStr . '_afternoon_time_from'] = '01/01/2000 ' . $data[$dayStr . '_afternoon_time_from'][0] . ':' . $data[$dayStr . '_afternoon_time_from'][1] . ':' . $data[$dayStr . '_afternoon_time_from'][2];
                $objWorkingTimes[$dayStr . '_afternoon_time_to'] = '01/01/2000 ' . $data[$dayStr . '_afternoon_time_to'][0] . ':' . $data[$dayStr . '_afternoon_time_to'][1] . ':' . $data[$dayStr . '_afternoon_time_to'][2];
                $objWorkingTimes['closed_' . $dayStr . '_afternoon'] = $data['closed_' . $dayStr . '_afternoon'];
            }

            $model->setObjWorkingTimes($objWorkingTimes);

            $this->_eventManager->dispatch(
                'filoblu_storelocator_store_prepare_save',
                [
                    'store' => $model,
                    'request' => $this->getRequest()
                ]
            );

            try {
                $model->save();
                $this->messageManager->addSuccess(__('You saved this Store.'));
                $this->_objectManager->get('Magento\Backend\Model\Session')->setFormData(false);
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['store_id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/');
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\RuntimeException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the store.'));
            }

            $this->_getSession()->setFormData($data);
            return $resultRedirect->setPath('*/*/edit', ['store_id' => $this->getRequest()->getParam('store_id')]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    public function uploadFileAndGetName($input, $destinationFolder, $data)
    {
        try {
            if (isset($data[$input]['delete'])) {
                return '';
            } else {
                $uploader = $this->uploaderFactory->create(['fileId' => $input]);
                $uploader->setAllowRenameFiles(true);
                $uploader->setFilesDispersion(true);
                $uploader->setAllowCreateFolders(true);
                $result = $uploader->save($destinationFolder);
                return $result['file'];
            }
        } catch (\Exception $e) {
            if ($e->getCode() != \Magento\Framework\File\Uploader::TMP_NAME_EMPTY) {
                throw new FrameworkException($e->getMessage());
            } else {
                if (isset($data[$input]['value'])) {
                    return $data[$input]['value'];
                }
            }
        }
        return '';
    }
}
